# helper script that generates .h and .c files from graphics gale files

import sys
import os
import time
import xml.etree.ElementTree as ET

maplist = []
tilesets = set()

destination_folder = "exported\\tileattributes\\"
destination_path = os.path.realpath('.') + "\\" + destination_folder

from glob import glob
print "************** Exporting Tile Attributes **************"
filenames = glob('..\gamedata\GGAnimations\*.gal')

def getmaplist():
    global maplist
    maplist = glob('..\gamedata\maps\*.gmx')

def gettilesetfilename(tilesetnode):
    global tilesetfilename
    sourcefilename = tilesetnode.get('source')

def gettilesets():
    for map in maplist:
        tree = ET.parse(map)
        root = tree.getroot()
        tilesetnode = root.find('tileset')
        tilesetfilename = '..\\gamedata\\tilesets\\' + tilesetnode.get('source')

        #we don't process any tile sets with "background" in their name because
        #we don't need their tile attributes
        tempfilename = tilesetfilename.lower()
        if not 'background' in tempfilename:
            tilesets.add(tilesetfilename)

conversion = {'0':'TILE_NONE',
              '1':'TILE_SOLID',
              '2':'TILE_SEMI_SOLID',
              '3':'TILE_SLOPE | TILE_45DEGREES_SLOPE',
              '4':'TILE_SLOPE | TILE_30DEGREES_SLOPE_TOP',
              '5':'TILE_SLOPE | TILE_30DEGREES_SLOPE_BOTTOM'
              }

def exportcfile(tilesetname, destinationfilename, convertedtileattributes):
    exported = open(destinationfilename, "w")
    exported.write("// **************************************************** \n")
    exported.write("// This file is automatically generated by\n")
    exported.write("// exportTileAttributes.py\n")
    exported.write("// **************************************************** \n\n")

    exported.write('#include "' + tilesetname + 'TileAttributes.h"\n\n')
    exported.write('#include "..\..\Collisions.h"\n')
    exported.write('\n')

    exported.write('unsigned short const ' + tilesetname + 'TileAttributes[' + str(len(convertedtileattributes)) + '] = \n')
    exported.write('{\n')

    for tileattribute in convertedtileattributes:
        exported.write('    ' + tileattribute + ',\n')
    exported.write('};\n\n')
    exported.close()

def exporthfile(tilesetname, destinationfilename, tileattributescount):
    exported = open(destinationfilename, "w")
    exported.write("// **************************************************** \n")
    exported.write("// This file is automatically generated by\n")
    exported.write("// exportTileAttributes.py\n")
    exported.write("// **************************************************** \n\n")
    exported.write('#ifndef ' + tilesetname.upper() + '_TILE_ATTRIBUTES_INCLUDE_H\n')
    exported.write('#define ' + tilesetname.upper() + '_TILE_ATTRIBUTES_INCLUDE_H\n\n')
    exported.write('extern unsigned short const ' + tilesetname + 'TileAttributes[' + str(tileattributescount)    + '];\n')
    exported.write('\n#endif\n')
    exported.close()  


def exporttileattributes(tileset):
    tilesetshortfilename = tileset.replace('/', '\\')
    tilesetshortfilename = tilesetshortfilename.rsplit('\\', 1)[1]
    tilesetname = tilesetshortfilename.split('.')[0]

    print "    Exporting " + tilesetshortfilename + "'s tile attributes"

    tree = ET.parse(tileset)
    root = tree.getroot()
    tileattributesnode = root.find('tileattributes')

    tileattributestext = tileattributesnode.text.strip('\n')

    tileattributes = tileattributestext.split(',')
    convertedtileattributes = []

    for tileattribute in tileattributes:
        convertedtileattributes.append(conversion[tileattribute])

    destinationc = destination_path + tilesetname + "TileAttributes.c"
    destinationh = destination_path + tilesetname + "TileAttributes.h"

    exportcfile(tilesetname, destinationc, convertedtileattributes)
    exporthfile(tilesetname, destinationh, len(convertedtileattributes))

def processtilesets():
    if not os.path.exists(destination_path):
        os.makedirs(destination_path)

    for tileset in tilesets:
        tilesettimestamp = os.path.getmtime(tileset)

        filename = tileset.replace('/', '\\')
        filename = filename.rsplit('\\', 1)[1]
        tilesetname = filename.split('.')[0]
        destinationc = destination_path + tilesetname + "TileAttributes.c"
        destinationh = destination_path + tilesetname + "TileAttributes.h"

        regenerate = False

        if not os.path.exists(destinationc):
            regenerate = True
        else:
            timestampc = os.path.getmtime(destinationc)
            if tilesettimestamp > timestampc:
                regenerate = True

        if not os.path.exists(destinationh):
            regenerate = True
        else:
            timestamph = os.path.getmtime(destinationh)
            if tilesettimestamp > timestamph:
                regenerate = True

        if regenerate:
            exporttileattributes(tileset)

def generatealltileattributesheader():

    exported = open("exported\\ExportedTileAttributes.h", "w")
    exported.write("// **************************************************** \n")
    exported.write("// This file is automatically generated by\n")
    exported.write("// export_tileattributes.py\n")
    exported.write("// **************************************************** \n\n")
    exported.write('#ifndef EXPORTED_TILE_ATTRIBUTES_H\n')
    exported.write('#define EXPORTED_TILE_ATTRIBUTES_H\n\n')

    for tileset in tilesets:
        filename = tileset.replace('/', '\\')
        filename = filename.rsplit('\\', 1)[1]
        tilesetname = filename.split('.')[0]
        exported.write('#include "tileattributes\\' + tilesetname + 'TileAttributes.h"\n')

    exported.write('\n#endif\n')
    exported.close()
    




getmaplist()
gettilesets()
processtilesets()
generatealltileattributesheader()