#include "HUDManager.h"

#include <genesis.h>
#include <KDebug.h>
#include <vdp_spr.h>
#include <vdp_tile.h>

#include "Collisions.h"
#include "Debug.h"
#include "generated\DebugSettings.h"
#include "GameFont.h"
#include "LevelManager.h"
#include "MathUtils.h"
#include "PlayerManager.h"
#include "ObjectManager.h"
#include "exported\ExportedGenResSprites.h"
#include "VDP_Extra.h"

u16 playerTileAttr;
u16 coinTileAttr;
u16 healthBarTopAttr;
u16 healthBarBottomAttr;
u32 oldFPS;

#define SHOW_LIVES_TIME 160

u16 showLivesTimer = 0;


SpriteDef playerIconSprite;
SpriteDef coinIconSprite;
SpriteDef healthBarSprite;

// General sprite drawing function.
void DrawString(u16 x, u16 y, const char* string, u16 stringLength)
{
    u16 loop = 0;
    while (stringLength--)
    {
        MyVDP_setSprite(x + (loop << 3), 
                        y, 
                        SPRITE_SIZE(1,1), 
                        TILE_ATTR_FULL(PAL0, 1, 0, 0, TILE_FONTINDEX + string[loop] - 32));
        loop++;
    }
}

inline void DrawLives()
{
    s16 x = playerObject->spritedef.posx;
    s16 y = playerObject->spritedef.posy - 24;
    u8 spriteSize = SPRITE_SIZE(2,2);
    u16 tile_attr = TILE_ATTR_FULL(PAL0, 1, 0, 0, TILE_FONTINDEX);

    MyVDP_setSprite(x - 24, y, spriteSize, TILE_ATTR_FULL(PAL0, 1, 0, 0, penguinHeadVDPInfo.startIndex));

    spriteSize = SPRITE_SIZE(1, 1);
    y += 4;

    u16 lives = playerData.lives;
    u16 n1 = lives % 10;
    lives /= 10;
    u16 n2 = lives % 10;

    MyVDP_setSprite(x - 8, y, spriteSize, tile_attr + 'x' - 32);
    MyVDP_setSprite(x, y, spriteSize, tile_attr + n2 + 16);
    MyVDP_setSprite(x + 8, y, spriteSize, tile_attr + n1 + 16);
}


inline void DrawNumberOfObjects()
{
    extern u16 numActiveObjects;

    u16 fps = numActiveObjects;
    u16 n1 = fps % 10;
    fps /= 10;
    u16 n2 = fps % 10;
    fps /= 10;
    u16 n3 = fps % 10;

    u16 tile_attr = TILE_ATTR_FULL(PAL0, 1, 0, 0, TILE_FONTINDEX + 16);
    u8 spriteSize = SPRITE_SIZE(1, 1);

    MyVDP_setSprite(48, 8, spriteSize, tile_attr + n3);
    MyVDP_setSprite(56, 8, spriteSize, tile_attr + n2);
    MyVDP_setSprite(64, 8, spriteSize, tile_attr + n1);
}

inline void DrawFPSString()
{
    if (debugSettings.showFPSCounter == FALSE)
    {
        return;
    }

    u16 fps = getFPS();
    u16 n1 = fps % 10;
    fps /= 10;
    u16 n2 = fps % 10;
    fps /= 10;
    u16 n3 = fps % 10;

    u16 tile_attr = TILE_ATTR_FULL(PAL0, 1, 0, 0, TILE_FONTINDEX + 16);
    u8 spriteSize = SPRITE_SIZE(1, 1);

    MyVDP_setSprite(144, 8, spriteSize, tile_attr + n3);
    MyVDP_setSprite(152, 8, spriteSize, tile_attr + n2);
    MyVDP_setSprite(160, 8, spriteSize, tile_attr + n1);
}


const u16 healthToSpriteIndex[9] =
{
    0,
    1 << 2,
    2 << 2,
    3 << 2,
    4 << 2,   
    4 << 2,        
    4 << 2,
    4 << 2,
    4 << 2
};

void HUDManager_Draw()
{

    // update lives string.
    if (showLivesTimer > 0)
    {
        DrawLives();
        showLivesTimer--;
    }

    if (playerData.invincibilityTimer > 0)
    {
        healthBarSprite.tile_attr = TILE_ATTR_FULL(PAL0, 1, 0, 0, heartVDPInfo.startIndex + (playerObject->healthPoints << 2));
        healthBarSprite.posx = playerObject->spritedef.posx - 16;
        healthBarSprite.posy = playerObject->spritedef.posy - 8;

        MyVDP_setSprite(healthBarSprite.posx,
                        healthBarSprite.posy,
                        healthBarSprite.size,
                        healthBarSprite.tile_attr);
    }

//    DrawNumberOfObjects();

    DrawFPSString();
}

void HUDManager_Init()
{
    // init lives string.
    HUDManager_UpdateLives();

    // init health bar.
    HUDManager_UpdateHealth();
    healthBarSprite.size = SPRITE_SIZE(4,1);

    oldFPS = 0;
}

void HUDManager_UpdateHealth()
{
}

void HUDManager_UpdateLives()
{
    showLivesTimer = SHOW_LIVES_TIME;
}
