#include "VDPManager.h"

#include <vdp_spr.h>

#include "VDP_Extra.h"

u16 tileIndex;

void VDPManager_ResetVDPInfo()
{
    tileIndex = 1;

    //ResetExportedSpriteVDPInfo();
}

void VDPManager_ReserveVDPArea(VDPInfo* vdpInfo, u16 numTiles)
{
    vdpInfo->startIndex = tileIndex;
    vdpInfo->numTiles = numTiles;
    tileIndex += numTiles;
}

void VDPManager_CopyGGAnimationFrameToVDP(const GGAnimation* animation, u16 frameNumber, const VDPInfo* vdpInfo)
{
    const GGFrame* frame = animation->frames[frameNumber];
    const GGSprite* sprite = frame->sprites[0];

    VDP_loadTileData(animation->allSpriteData + (sprite->VDPTileOffset << 3), 
                     vdpInfo->startIndex, 
                     animation->maxNumberOfTilesPerFrame,
                     1);
    VDP_waitDMACompletion();
}

void VDPManager_LoadSpriteDataToVDP(const struct genresSprites* sprite, VDPInfo* spriteVDPInfo)
{
    u16 numTiles = (sprite->width >> 3) * (sprite->height >> 3) * sprite->count;

    spriteVDPInfo->startIndex = tileIndex;
    spriteVDPInfo->numTiles = numTiles;
    tileIndex += numTiles;

    VDP_loadTileData( sprite->sprites[0], spriteVDPInfo->startIndex, spriteVDPInfo->numTiles, 1);
    VDP_waitDMACompletion();
}

void VDPManager_LoadGGAnimationToVDP(const GGAnimation* animation, VDPInfo* animationVDPInfo)
{
    animationVDPInfo->startIndex = tileIndex;
    animationVDPInfo->numTiles = animation->totalTiles;
    tileIndex += animationVDPInfo->numTiles;

    VDP_loadTileData(animation->allSpriteData, animationVDPInfo->startIndex, animation->totalTiles, 1);
    VDP_waitDMACompletion();
}

void VDPManager_ResetDisplay()
{
    VDP_setReg(0x0b, 0x00); // set scroll mode to plane/plane
    VDP_resetSprites();

    VDP_setPlanSize(64, 64);

    VDP_setVerticalScroll(PLAN_A, 0);
    VDP_setVerticalScroll(PLAN_B, 0);
    VDP_setHorizontalScroll(PLAN_A, 0);
    VDP_setHorizontalScroll(PLAN_B, 0);

    VDP_updateSprites();

    VDP_fillTileMapRect(APLAN, 0, 0, 0, 64, 64);
    VDP_fillTileMapRect(BPLAN, 0, 0, 0, 64, 64);

    VDP_setTextPalette(0);
}

