#include "VDPPlaneManager_HorizontalMapFunctions.h"

#include "Config.h"
#include "Debug.h"
#include "LevelManager.h"
#include "MathUtils.h"
#include "VDP_Extra.h"
#include "PlayerManager.h"
#include "ScrollManager.h"

extern u16 vdpWidthMinusOne;
extern DMADataStruct* dmaDataStruct;
extern ScrollDataStruct* scrollData;
extern VPMap* map;
extern u32 plan;
extern u16 palette;
extern u16 vdpPlaneWidth;

static void PrepareMapDMAData(u16 mapColumn)
{
    dmaDataStruct->mapDMADestinationAddress = plan + ((mapColumn & vdpWidthMinusOne) << 1); 

    u16 tileInfo = TILE_ATTR_FULL(palette, 0, 0, 0, map->tilesetVDPInfo->startIndex);

    // fill the map column dma data
    u16 loop;
    u16* mapDMAData = dmaDataStruct->mapDMAData;
    const u16* mapLocation = (map->tilemap + mapColumn);

    for (loop = map->height; loop > 0; loop--)
    {
        *(mapDMAData) = *(mapLocation) + tileInfo;
        mapDMAData++;
        mapLocation += map->width;
    }
}

void VDPPlaneManager_HorizontalMap_Update()
{
    // check current tilePositionX
    u16 tilePositionX = 0;
    dmaDataStruct->updateMap = 0;
    
    if (PIXEL_TO_TILE(scrollData->scrollx_vdp) - 1 > 0)
    {
        tilePositionX = (u16)PIXEL_TO_TILE(scrollData->scrollx_vdp) - 1;
    }

    // if it's different from the old one, we'll have to handle updating
    // the plane.
    if (dmaDataStruct->oldTilePosition != tilePositionX)
    {
        u16 newTilePositionX = tilePositionX;

        // here we check the actual position difference of the player
        // since the last frame. Checking the player's x speed is unreliable
        // because it can be set to 0 when hitting a wall.
        s16 speedx = playerObject->x - playerData.oldPositionX;

        if (speedx > 0)
        {
            // by default the position of the left most tile column is calculated.
            // if the plane is scrolling right, we use the index of the 
            // left most tile column for our drawing destination.

            newTilePositionX += TILE_UPDATE_AREA_WIDTH; 

            if (newTilePositionX > map->width - 1)
            {
                newTilePositionX = map->width - 1;
            }
        }

        // we're assuming we're only updating one column at a time. If the player
        // goes faster than 8 pixels in a frame, we'll miss columns.
        PrepareMapDMAData(newTilePositionX);

        dmaDataStruct->updateMap = 1;
    }

    dmaDataStruct->oldTilePosition = tilePositionX;
}

void VDPPlaneManager_HorizontalMap_UpdateVDP()
{
    if (dmaDataStruct->updateMap == 1)
    {
        MyVDP_doDMA(VDP_DMA_VRAM, 
                   (u32)dmaDataStruct->mapDMAData, 
                   dmaDataStruct->mapDMADestinationAddress, 
                   dmaDataStruct->mapDMADataCopyLength, 
                   vdpPlaneWidth << 1);
        VDP_waitDMACompletion();
    }
}


static void DrawColumnDMA(u16 mapColumn)
{
    dmaDataStruct->mapDMADestinationAddress = plan + ((mapColumn & vdpWidthMinusOne) << 1); 

    // fill the map row dma data
    u16 loop;
    for (loop = 0; loop < map->height; loop++)
    {
        dmaDataStruct->mapDMAData[loop] = map->tilemap[mapColumn + (loop * map->width)] + TILE_ATTR_FULL(palette, 0, 0, 0, map->tilesetVDPInfo->startIndex);
    }

    MyVDP_doDMA(VDP_DMA_VRAM, 
               (u32)dmaDataStruct->mapDMAData, 
               dmaDataStruct->mapDMADestinationAddress, 
               dmaDataStruct->mapDMADataCopyLength, 
               vdpPlaneWidth << 1);

    VDP_waitDMACompletion();
}


void VDPPlaneManager_HorizontalMap_SetupPlane()
{
    s16 tilePositionX = 0;
    
    if (PIXEL_TO_TILE(scrollData->scrollx_vdp) - 1 > 0)
    {
        tilePositionX = PIXEL_TO_TILE(scrollData->scrollx_vdp) - 1;
    }

    dmaDataStruct->oldTilePosition = tilePositionX;

    // compute start and end tiles
    s16 starttilex = tilePositionX;
    s16 endtilex = (tilePositionX + TILE_UPDATE_AREA_WIDTH); 

    if (endtilex > map->width - 1)
    {
        endtilex = map->width - 1;
    }

    s16 loop = 0;
    for (loop = starttilex; loop <= endtilex; loop++)
    {
        DrawColumnDMA(loop);
    }
}

