#include "Boss.h"

#include "ObjectCommon.h"

//#define SKIP_INTRO
#define HEALTH_POINTS 35
#define MOVE_HEALTH_THRESHOLD 30
#define MOVE_FASTER_HEALTH_THRESHOLD 8

const u16 explosionPointsX[] =
{
    PIXEL_TO_VIRTUAL(70),
    PIXEL_TO_VIRTUAL(96),
    PIXEL_TO_VIRTUAL(48),
    PIXEL_TO_VIRTUAL(32),
    PIXEL_TO_VIRTUAL(64),
    PIXEL_TO_VIRTUAL(112),
    PIXEL_TO_VIRTUAL(48),
    PIXEL_TO_VIRTUAL(32),
    PIXEL_TO_VIRTUAL(64),
    PIXEL_TO_VIRTUAL(112)
};

const u16 explosionPointsY[] = 
{
    PIXEL_TO_VIRTUAL(30),
    PIXEL_TO_VIRTUAL(120),
    PIXEL_TO_VIRTUAL(100),
    PIXEL_TO_VIRTUAL(60),
    PIXEL_TO_VIRTUAL(140),
    PIXEL_TO_VIRTUAL(80),
    PIXEL_TO_VIRTUAL(50),
    PIXEL_TO_VIRTUAL(110),
    PIXEL_TO_VIRTUAL(160),
    PIXEL_TO_VIRTUAL(70)
};

void BossInit(ObjectStruct* object);
void BossIntro(ObjectStruct* object);
void BossUpdate(ObjectStruct* object);
void BossDeath(ObjectStruct* object);
void BossPostDeathWait(ObjectStruct* object);

void BossDestroy(ObjectStruct* object);
void BossDraw(ObjectStruct* object);
void BossCollide(ObjectStruct* object, ObjectStruct* collidedObject);


ObjectStruct* BossCreate(s16 x, s16 y)
{
#ifdef SKIP_INTRO
    ObjectStruct* object = ObjectManager_CreateObject(TYPE_ENEMY,
                                                      BossUpdate, 
                                                      BossDestroy, 
                                                      BossDraw,
                                                      BossCollide);
#else
    ObjectStruct* object = ObjectManager_CreateObject(TYPE_ENEMY,
                                                      BossInit, 
                                                      BossDestroy, 
                                                      BossDraw,
                                                      BossCollide);
#endif

    if (!object)
    {
#ifdef DEBUG_OBJECTS
        KDebug_Alert("enemy could not be created!");
#endif
        return NULL;
    }
#ifdef SKIP_INTRO
    object->x                           = PIXEL_TO_VIRTUAL(160);
#else
    object->x                           = x;
#endif
    object->y                           = y;
    object->speedx                      = -1;
    object->speedy                      = 0;
    object->animation                   = &Boss;
    object->aiState                     = ENEMY_AI_STATE_ACTIVE;
    object->rectLeft                    = PIXEL_TO_VIRTUAL(8);
    object->rectRight                   = PIXEL_TO_VIRTUAL(BossTracks.width);
    object->rectTop                     = PIXEL_TO_VIRTUAL(0);
    object->rectBottom                  = PIXEL_TO_VIRTUAL(176);
    object->healthPoints                = HEALTH_POINTS;
    object->damage                      = 1;
    object->visible                     = 1;
    object->animationFrame              = 0;
    object->animationSubFrameCounter    = 0;
    object->invulnerableTimer           = 0;
    ComputeStandardActiveArea(object);

#ifdef DEBUG_OBJECTS
    KDebug_Alert("enemyCreate");
#endif
    
    return object;
}

void BossInit(ObjectStruct* object)
{
    playerData.disablePlayer = 1;
    object->speedx = 2;

    ScrollManager_SetCameraShakeTime(320);

    object->updateFunction = BossIntro;
}

void BossIntro(ObjectStruct* object)
{
    object->x -= object->speedx;

    if (object->x == PIXEL_TO_VIRTUAL(160))
    {
        object->speedx = 0;
        object->updateFunction = BossUpdate;
        playerData.disablePlayer = 0;
    }

    object->spritedef.posx = VIRTUAL_TO_PIXEL(object->x) - foregroundScrollData.scrollx_vdp;
    object->spritedef.posy = VIRTUAL_TO_PIXEL(object->y) - foregroundScrollData.scrolly_vdp;
}

void BossUpdate(ObjectStruct* object)
{
    if (object->invulnerableTimer > 0)
    {
        object->invulnerableTimer--;
    }

    if ((object->lifetime & 127) == 0)
    {
        ExplosionCreate(object->x+ PIXEL_TO_VIRTUAL(24), object->y + PIXEL_TO_VIRTUAL(80));
        EnemyCannonBallCreate(object->x + PIXEL_TO_VIRTUAL(16), object->y + PIXEL_TO_VIRTUAL(70));
    }

    object->x += object->speedx;

    if (VIRTUAL_TO_PIXEL(object->x) < 64)
    {
        object->speedx = -object->speedx;
    }
    else if (VIRTUAL_TO_PIXEL(object->x) > 160)
    {
        object->speedx = -object->speedx;
    }

    object->spritedef.posx = VIRTUAL_TO_PIXEL(object->x) - foregroundScrollData.scrollx_vdp;
    object->spritedef.posy = VIRTUAL_TO_PIXEL(object->y) - foregroundScrollData.scrolly_vdp;

}

void BossDeath(ObjectStruct* object)
{
    object->y += object->speedy >> 4;

    object->speedy++;

    if (object->visible)
    {
        if (object->lifetime % 2)
        {
            u16 index = (object->lifetime >> 1) % 10;
            ExplosionCreate(object->x + explosionPointsX[index], object->y + explosionPointsY[index]);
        }
    }
    else
    {
        object->lifetime = 0;
        object->updateFunction = BossPostDeathWait;
    }

    object->spritedef.posx = VIRTUAL_TO_PIXEL(object->x) - foregroundScrollData.scrollx_vdp;
    object->spritedef.posy = VIRTUAL_TO_PIXEL(object->y) - foregroundScrollData.scrolly_vdp;
}

void BossPostDeathWait(ObjectStruct* object)
{
    if (object->lifetime > 300) // five seconds
    {
        DoorManager_ExitLevel();
        object->objectState = OBJECTSTATE_INACTIVE;
    }
}

void BossDestroy(ObjectStruct* object)
{
#ifdef DEBUG_OBJECTS
    KDebug_Alert("BossDestroy");
#endif
}

void BossDraw(ObjectStruct* object)
{
    u16 palette = PAL1;

    if (object->speedx == 0)
    {
        object->animationFrame = 0;
    }

    object->animationSubFrameCounter++;


    if (object->healthPoints <= MOVE_FASTER_HEALTH_THRESHOLD)
    {
        object->animationSubFrameCounter++; // twice as fast
    }

    if (object->invulnerableTimer > 0 &&
        object->invulnerableTimer < 5)
    {
        palette = PAL0;
    }

    //ComplexAnimation_Draw(object->spritedef.posx, object->spritedef.posy, &boss, 0, 0, PAL1);

    GGAnimation_Draw(object->spritedef.posx + 56, object->spritedef.posy + 8, &BossHead, 0, 0, palette);
    GGAnimation_Draw(object->spritedef.posx + 24, object->spritedef.posy + 56, &BossBody, 0, 0, palette);
    GGAnimation_Draw(object->spritedef.posx, object->spritedef.posy + 88, &BossHand, 0, 0, palette);
    GGAnimation_Draw(object->spritedef.posx + 40, object->spritedef.posy + 120, &BossBottomRing, 0, 0, palette);
    GGAnimation_Draw(object->spritedef.posx + 8, object->spritedef.posy + 136, &BossTracks, object->animationFrame, 0, palette);

    if (object->animationSubFrameCounter > BossTracks.frameDelayTime)
    {
        object->animationFrame++;
        object->animationSubFrameCounter = 0;
    }

    if (object->animationFrame > BossTracks.numberOfFrames - 1)
    {
        object->animationFrame = 0;
    }
}

void BossCollide(ObjectStruct* object, ObjectStruct* collidedObject)
{
    if (object->invulnerableTimer > 0)
    {
        return;
    }

    if (object->aiState != ENEMY_AI_STATE_ACTIVE)
    {
        return;
    }

    if (collidedObject->objectType != TYPE_PLAYER_PROJECTILE)
    {
        return;
    }

    object->healthPoints -= collidedObject->damage;

    object->invulnerableTimer = 10;

    if (object->healthPoints <= 0)
    {
        object->updateFunction = BossDeath;
        object->aiState = ENEMY_AI_STATE_DEAD;
        object->speedy    = 8;
        ObjectManager_ResetEnemyProjectileCollection();
    }

    if (object->healthPoints == MOVE_HEALTH_THRESHOLD &&
        object->speedx == 0)
    {
        object->speedx = 1;
    }

    if (object->healthPoints == MOVE_FASTER_HEALTH_THRESHOLD)
    {
        object->speedx *= 2;
        VDP_setPalette(PAL1, bossRedPaletteSprite.pal);
    }
}

