#include "Coin.h"

#include "ObjectCommon.h"
#include "..\HUDManager.h"

ObjectStruct* CoinCreate(s16 x, s16 y)
{
    ObjectStruct* object = ObjectManager_CreateObject(TYPE_PICKUP,
                                                      CoinUpdate, 
                                                      CoinDestroy, 
                                                      CoinDraw,
                                                      CoinCollide);

    if (!object)
    {
#ifdef DEBUG_OBJECTS
        KDebug_Alert("Coin could not be created!");
#endif
        return NULL;
    }

    object->x                           = x;
    object->y                           = y;
    object->animation                   = &Coin;
    object->speedx                      = 0;
    object->speedy                      = 0;
    object->rectLeft                    = 0;
    object->rectRight                   = PIXEL_TO_VIRTUAL(16);
    object->rectTop                     = 0;
    object->rectBottom                  = PIXEL_TO_VIRTUAL(16);
    object->healthPoints                = 1;
    object->damage                      = 0;
    object->animationFrame              = 0;
    object->animationSubFrameCounter    = 0;
    object->direction                   = 0;
    object->palette                     = PAL0;
    ComputeStandardActiveArea(object);

#ifdef DEBUG_OBJECTS
    KDebug_Alert("CoinCreate");
#endif

    return object;
}

void CoinUpdate(ObjectStruct* object)
{
    //KDebug_Alert("Coin Update");

    object->spritedef.posx = VIRTUAL_TO_PIXEL(object->x) - foregroundScrollData.scrollx_vdp;
    object->spritedef.posy = VIRTUAL_TO_PIXEL(object->y) - foregroundScrollData.scrolly_vdp;
}

void CoinDestroy(ObjectStruct* object)
{
#ifdef DEBUG_OBJECTS
    KDebug_Alert("CoinDestroy");
#endif
}

void CoinDraw(ObjectStruct* object)
{
    object->animationSubFrameCounter++;

    if (object->animationSubFrameCounter > object->animation->frameDelayTime)
    {
        object->animationFrame++;
        object->animationSubFrameCounter = 0;
    }

    if (object->animationFrame > object->animation->numberOfFrames - 1)
    {
        object->animationFrame = 0;
    }

    GGAnimation_DrawObject(object);
}

void CoinCollide(ObjectStruct* object, ObjectStruct* collidedObject)
{
    object->objectState = OBJECTSTATE_INACTIVE;

    playerData.numCoins++;

    if (playerData.numCoins > 999)
    {
        playerData.numCoins = 999;
    }
}
