#include "Effect.h"

#include "ObjectCommon.h"

void EffectUpdate(ObjectStruct* object);
void EffectDestroy(ObjectStruct* object);


u16 EffectCreate(const GGAnimation* animation, const VDPInfo* vdpInfo, u16 x, u16 y, u16 direction)
{
    ObjectStruct* object = ObjectManager_CreateObject(TYPE_EFFECT,
                                                 EffectUpdate, 
                                                 EffectDestroy, 
                                                 GGAnimation_DrawObject,
                                                 CollisionFunctionStub);

    if (!object)
    {
#ifdef DEBUG_OBJECTS
        KDebug_Alert("Effect could not be created!");
#endif
        return 0;
    }

    object->x                           = x;
    object->y                           = y;
    object->animation                   = animation;
    object->animationFrame              = 0;
    object->animationSubFrameCounter    = 0;
    object->direction                   = direction;
    object->palette                     = PAL0;
    ComputeStandardActiveArea(object);

#ifdef DEBUG_OBJECTS
    KDebug_Alert("EffectCreate");
#endif

    return 1;
}

void EffectUpdate(ObjectStruct* object)
{
    //KDebug_Alert("Effect Update");

    object->animationSubFrameCounter++;

    if (object->animationSubFrameCounter > object->animation->frameDelayTime)
    {
        object->animationFrame++;
        object->animationSubFrameCounter = 0;
    }

    if (object->animationFrame > object->animation->numberOfFrames - 1)
    {
        object->objectState = OBJECTSTATE_INACTIVE;
        return;
    }

    object->spritedef.posx = VIRTUAL_TO_PIXEL(object->x) - foregroundScrollData.scrollx_vdp;
    object->spritedef.posy = VIRTUAL_TO_PIXEL(object->y) - foregroundScrollData.scrolly_vdp;
}

void EffectDestroy(ObjectStruct* object)
{
#ifdef DEBUG_OBJECTS
    KDebug_Alert("EffectDestroy");
#endif
}
