#include "EnemyBullet.h"

#include "ObjectCommon.h"

void EnemyBulletUpdate(ObjectStruct* object);
void EnemyBulletDestroy(ObjectStruct* object);
void EnemyBulletDraw(ObjectStruct* object);
void EnemyBulletCollide(ObjectStruct* object, ObjectStruct* collidedObject);

ObjectStruct* EnemyBulletCreate(s16 x, s16 y, s16 speedx, s16 speedy)
{
    ObjectStruct* object = ObjectManager_CreateObject(TYPE_ENEMY_PROJECTILE,
                                                   EnemyBulletUpdate, 
                                                   EnemyBulletDestroy, 
                                                   EnemyBulletDraw,
                                                   EnemyBulletCollide);

    if (!object)
    {
#ifdef DEBUG_OBJECTS
        KDebug_Alert("EnemyBullet could not be created!");
#endif
        return NULL;
    }

    object->x                 = x;
    object->y                 = y;
    object->speedx            = speedx;
    object->speedy            = speedy;
    object->spritedef.size       = SPRITE_SIZE(1,1);
    object->spritedef.tile_attr  = TILE_ATTR_FULL(PAL1,1,0,0,bulletVDPInfo.startIndex);
    object->rectLeft          = PIXEL_TO_VIRTUAL(-4);
    object->rectRight         = PIXEL_TO_VIRTUAL(4);
    object->rectTop           = PIXEL_TO_VIRTUAL(-4);
    object->rectBottom        = PIXEL_TO_VIRTUAL(4);
    object->damage            = 1;
    object->healthPoints      = 1;
    object->visible           = 1;
    object->aiState           = ENEMY_AI_STATE_ACTIVE;
    ComputeStandardActiveArea(object);

#ifdef DEBUG_OBJECTS
    KDebug_Alert("EnemyBulletCreate");
#endif

    return object;
}

void EnemyBulletUpdate(ObjectStruct* object)
{
    //KDebug_Alert("EnemyBullet Update");
    object->x += object->speedx;
    object->y += object->speedy;

    object->spritedef.posx = VIRTUAL_TO_PIXEL(object->x) - foregroundScrollData.scrollx_vdp - 4;
    object->spritedef.posy = VIRTUAL_TO_PIXEL(object->y) - foregroundScrollData.scrolly_vdp - 4;

    if (object->visible)
    {
        if (IsTileSolid_Virtual(object->x, object->y))
        {
            object->objectState = OBJECTSTATE_INACTIVE;
            ExplosionCreate(object->x, object->y);
        }
    }
    else
    {
        object->objectState = OBJECTSTATE_INACTIVE;
    }
}

void EnemyBulletDestroy(ObjectStruct* object)
{
#ifdef DEBUG_OBJECTS
    KDebug_Alert("EnemyBulletDestroy");
#endif
}

void EnemyBulletDraw(ObjectStruct* object)
{
    MyVDP_setSpriteDef(object->spritedef);
}

void EnemyBulletCollide(ObjectStruct* object, ObjectStruct* collidedObject)
{
    object->objectState = OBJECTSTATE_INACTIVE;
}

