#include "EnemyCannonBall.h"

#include "ObjectCommon.h"

#define ENEMY_AI_STATE_LAUNCHING    1
#define ENEMY_AI_STATE_ROLLING      2

ObjectStruct* EnemyCannonBallCreate(s16 x, s16 y)
{
    ObjectStruct* object = ObjectManager_CreateObject(TYPE_ENEMY_PROJECTILE,
                                                   EnemyCannonBallUpdate, 
                                                   EnemyCannonBallDestroy, 
                                                   EnemyCannonBallDraw,
                                                   EnemyCannonBallCollide);

    if (!object)
    {
#ifdef DEBUG_OBJECTS
        KDebug_Alert("EnemyCannonBall could not be created!");
#endif
        return NULL;
    }

    object->x                           = x;
    object->y                           = y;
    object->speedx                      = -4;
    object->speedy                      = 0;
    object->animation                   = &EnemyCannonBall;
    object->spritedef.size              = SPRITE_SIZE(3,3);
    object->spritedef.tile_attr         = TILE_ATTR_FULL(PAL1,1,0,0,EnemyCannonBallVDPInfo.startIndex);
    object->rectLeft                    = PIXEL_TO_VIRTUAL(0);
    object->rectRight                   = PIXEL_TO_VIRTUAL(EnemyCannonBall.width);
    object->rectTop                     = PIXEL_TO_VIRTUAL(0);
    object->rectBottom                  = PIXEL_TO_VIRTUAL(EnemyCannonBall.height);
    object->damage                      = 1;
    object->healthPoints                = 1;
    object->visible                     = 1;
    object->animationFrame              = 0;
    object->animationSubFrameCounter    = 0;
    object->aiState                     = ENEMY_AI_STATE_LAUNCHING;
    object->direction                   = 0;
    object->palette                     = PAL1;
    ComputeStandardActiveArea(object);


#ifdef DEBUG_OBJECTS
    KDebug_Alert("EnemyCannonBallCreate");
#endif

    return object;
}

void EnemyCannonBallUpdate(ObjectStruct* object)
{
    //KDebug_Alert("EnemyCannonBall Update");
    object->x += object->speedx;
    object->y += object->speedy >> 2;

    object->speedy++;

    object->spritedef.posx = VIRTUAL_TO_PIXEL(object->x) - foregroundScrollData.scrollx_vdp;
    object->spritedef.posy = VIRTUAL_TO_PIXEL(object->y) - foregroundScrollData.scrolly_vdp;

    if (object->visible)
    {
        if (IsTileSolid_Virtual(object->x, object->y + object->rectBottom))
        {
            object->speedy = 0;
            object->aiState = ENEMY_AI_STATE_ROLLING;
        }
    }
    else
    {
        object->objectState = OBJECTSTATE_INACTIVE;
    }
}

void EnemyCannonBallDestroy(ObjectStruct* object)
{
#ifdef DEBUG_OBJECTS
    KDebug_Alert("EnemyCannonBallDestroy");
#endif
}

void EnemyCannonBallDraw(ObjectStruct* object)
{
    if (object->aiState == ENEMY_AI_STATE_ROLLING)
    {
        object->animationSubFrameCounter++;

        if (object->animationSubFrameCounter > object->animation->frameDelayTime)
        {
            object->animationFrame++;
            object->animationSubFrameCounter = 0;
        }

        if (object->animationFrame > object->animation->numberOfFrames - 1)
        {
            object->animationFrame = 0;
        }
    }

    GGAnimation_DrawObject(object);
}

void EnemyCannonBallCollide(ObjectStruct* object, ObjectStruct* collidedObject)
{
    object->objectState = OBJECTSTATE_INACTIVE;

    ExplosionCreate(object->x + PIXEL_TO_VIRTUAL(12), object->y + PIXEL_TO_VIRTUAL(12));
}

