#include "EnemyLarge.h"

#include "ObjectCommon.h"

void EnemyLargeUpdate(ObjectStruct* enemyObject);
void EnemyLargeDestroy(ObjectStruct* enemyObject);
void EnemyLargeCollide(ObjectStruct* enemyObject, ObjectStruct* collidedObject);


ObjectStruct* EnemyLargeCreate(s16 x, s16 y)
{
    ObjectStruct* object = ObjectManager_CreateObject(TYPE_ENEMY,
                                                      EnemyLargeUpdate, 
                                                      EnemyLargeDestroy, 
                                                      GGAnimation_DrawObject,
                                                      EnemyLargeCollide);

    if (!object)
    {
#ifdef DEBUG_OBJECTS
        KDebug_Alert("enemy could not be created!");
#endif
        return NULL;
    }

    object->x                      = x;
    object->y                      = y;
    object->speedx                 = 0;
    object->speedy                 = 0;
    object->animation              = &EnemyLarge;
    object->aiState                = ENEMY_AI_STATE_ACTIVE;
    object->rectLeft               = PIXEL_TO_VIRTUAL(0);
    object->rectRight              = PIXEL_TO_VIRTUAL(EnemyLarge.width);
    object->rectTop                = PIXEL_TO_VIRTUAL(0);
    object->rectBottom             = PIXEL_TO_VIRTUAL(EnemyLarge.height);
    object->healthPoints           = 1;
    object->damage                 = 1;
    object->visible                = 1;
    object->direction              = 0;
    object->palette                = PAL1;
    ComputeStandardActiveArea(object);

#ifdef DEBUG_OBJECTS
    KDebug_Alert("enemyCreate");
#endif
    
    return object;
}

void EnemyLargeUpdate(ObjectStruct* object)
{
    if (object->aiState == ENEMY_AI_STATE_ACTIVE)
    {
        object->x += object->speedx;
        object->y += object->speedy;
    }
    else if (object->aiState == ENEMY_AI_STATE_DEAD)
    {
        object->x += object->speedx;
        object->y += object->speedy >> 2;

        object->speedy++;

        if (object->visible)
        {
            if ((object->lifetime % 20) == 0)
            {
                ExplosionCreate(object->x + (PIXEL_TO_VIRTUAL(object->animation->width) >> 1), object->y);
            }
        }
        else
        {
            object->objectState = OBJECTSTATE_INACTIVE;
        }
    }

    object->spritedef.posx = VIRTUAL_TO_PIXEL(object->x) - foregroundScrollData.scrollx_vdp;
    object->spritedef.posy = VIRTUAL_TO_PIXEL(object->y) - foregroundScrollData.scrolly_vdp;
}

void EnemyLargeDestroy(ObjectStruct* object)
{
#ifdef DEBUG_OBJECTS
    KDebug_Alert("EnemyLargeDestroy");
#endif
}

void EnemyLargeCollide(ObjectStruct* object, ObjectStruct* collidedObject)
{
    if (object->aiState != ENEMY_AI_STATE_ACTIVE)
    {
        return;
    }

    if (collidedObject->objectType != TYPE_PLAYER_PROJECTILE)
    {
        return;
    }

    object->healthPoints -= collidedObject->damage;

    if (object->healthPoints <= 0)
    {
        ExplosionCreate(object->x, object->y);
        object->aiState   = ENEMY_AI_STATE_DEAD;
        object->speedy    = 8;
        CoinCreate(object->x, object->y);
    }
}
