#include "EnemySmall.h"

#include <KDebug.h>
#include <vdp_spr.h>
#include <vdp_tile.h>

#include "ObjectCommon.h"

void EnemySmallUpdate(ObjectStruct* enemyObject);
void EnemySmallDestroy(ObjectStruct* enemyObject);
void EnemySmallDraw(ObjectStruct* enemyObject);
void EnemySmallCollide(ObjectStruct* enemyObject, ObjectStruct* collidedObject);


const u16 upDownAnimation[8] =
{
    0,
    -2,
    -3,
    -2,
    0,
    2,
    3,
    2,
};

ObjectStruct* EnemySmallCreate(s16 x, s16 y)
{
    ObjectStruct* object = ObjectManager_CreateObject(TYPE_ENEMY,
                                                      EnemySmallUpdate, 
                                                      EnemySmallDestroy, 
                                                      EnemySmallDraw,
                                                      EnemySmallCollide);

    if (!object)
    {
#ifdef DEBUG_OBJECTS
        KDebug_Alert("enemy could not be created!");
#endif
        return NULL;
    }

    object->x                        = x;
    object->y                        = y; // y spawn position is at the bottom of the sprite
    object->speedx                   = 0;
    object->speedy                   = 0;
    object->animation                = &EnemySmall;
    object->aiState                  = ENEMY_AI_STATE_ACTIVE;
    object->rectLeft                 = PIXEL_TO_VIRTUAL(10);
    object->rectRight                = PIXEL_TO_VIRTUAL(EnemySmall.width - 10);
    object->rectTop                  = PIXEL_TO_VIRTUAL(10);
    object->rectBottom               = PIXEL_TO_VIRTUAL(EnemySmall.height - 8);
    object->healthPoints             = 1;
    object->damage                   = 1;
    object->visible                  = 1;
    object->animationFrame           = 1;
    object->animationSubFrameCounter = 0;
    object->extra1                   = 1;
    object->direction                = 0;
    object->palette                  = PAL1;

    ComputeStandardActiveArea(object);

#ifdef DEBUG_OBJECTS
    KDebug_Alert("enemyCreate");
#endif
    
    return object;
}

void EnemySmallUpdate(ObjectStruct* object)
{
    if (object->aiState == ENEMY_AI_STATE_ACTIVE)
    {
        object->x += object->speedx;
        object->y += object->speedy;
    }
    else if (object->aiState == ENEMY_AI_STATE_DEAD)
    {
        object->x += object->speedx;
        object->y += object->speedy >> 2;

        object->speedy++;

        if (object->visible)
        {
            if ((object->lifetime % 20) == 0)
            {
                ExplosionCreate(object->x + (PIXEL_TO_VIRTUAL(object->animation->width) >> 1), object->y);
            }
        }
        else
        {
            object->objectState = OBJECTSTATE_INACTIVE;
        }
    }

    object->spritedef.posx = VIRTUAL_TO_PIXEL(object->x) - foregroundScrollData.scrollx_vdp;
    object->spritedef.posy = VIRTUAL_TO_PIXEL(object->y) - foregroundScrollData.scrolly_vdp + upDownAnimation[object->extra1 & 7];
}

void EnemySmallDestroy(ObjectStruct* object)
{
#ifdef DEBUG_OBJECTS
    KDebug_Alert("EnemySmallDestroy");
#endif
}

void EnemySmallDraw(ObjectStruct* object)
{
    object->animationSubFrameCounter++;

    if (object->animationSubFrameCounter > object->animation->frameDelayTime)
    {
        object->animationFrame++;
        object->animationSubFrameCounter = 0;
    }

    if (object->animationFrame > object->animation->numberOfFrames - 1)
    {
        object->animationFrame = 0;
    }

    object->extra2++;

    if (object->extra2 > 10)
    {
        object->extra1++;
        object->extra2 = 0;
    }

    GGAnimation_DrawObject(object);
}

void EnemySmallCollide(ObjectStruct* object, ObjectStruct* collidedObject)
{
    if (object->aiState != ENEMY_AI_STATE_ACTIVE)
    {
        return;
    }

    if (collidedObject->objectType != TYPE_PLAYER_PROJECTILE)
    {
        return;
    }

    object->healthPoints -= collidedObject->damage;

    if (object->healthPoints <= 0)
    {
        ExplosionCreate(object->x, object->y);
        object->aiState   = ENEMY_AI_STATE_DEAD;
        object->speedy    = 8;
        CoinCreate(object->x + PIXEL_TO_VIRTUAL(object->animation->width >> 1), object->y);
    }
}
