#include "EnemyTallFall.h"

#include "Effect.h"
#include "ObjectCommon.h"


void EnemyTallFallUpdate(ObjectStruct* enemyObject);
void EnemyTallFallDestroy(ObjectStruct* enemyObject);
void EnemyTallFallDraw(ObjectStruct* enemyObject);
void EnemyTallFallCollide(ObjectStruct* enemyObject, ObjectStruct* collidedObject);

#define SHAKE_TIME (u16)30
#define PREFALL_TIME 30

#define ENEMY_AI_STATE_WAIT     1
#define ENEMY_AI_STATE_PREFALL  2
#define ENEMY_AI_STATE_FALL     3
#define ENEMY_AI_STATE_LANDED   4


s16 RayCast(s16 x, s16 y)
{
    s16 tilex = VIRTUAL_TO_TILE(x);
    s16 starty = VIRTUAL_TO_TILE(y);
    s16 endy = currentForegroundMap->height;

    s16 loop;
    for (loop = starty; loop < endy; loop++)
    {
        if ((IsTileSolid_Tile(tilex, loop) != 0))
        {
            return TILE_TO_VIRTUAL(loop);
        }
    }

    return -1;
}


ObjectStruct* EnemyTallFallCreate(s16 x, s16 y)
{
    ObjectStruct* object = ObjectManager_CreateObject(TYPE_ENEMY,
                                                      EnemyTallFallUpdate, 
                                                      EnemyTallFallDestroy, 
                                                      EnemyTallFallDraw,
                                                      EnemyTallFallCollide);

    if (!object)
    {
#ifdef DEBUG_OBJECTS
        KDebug_Alert("enemy could not be created!");
#endif
        return NULL;
    }

    object->x                           = x;
    object->y                           = y;
    object->speedx                      = 0;
    object->speedy                      = 0;
    object->animation                   = &EnemyTallFall;
    object->aiState                     = ENEMY_AI_STATE_WAIT;
    object->rectLeft                    = PIXEL_TO_VIRTUAL(8);
    object->rectRight                   = PIXEL_TO_VIRTUAL(31);
    object->rectTop                     = PIXEL_TO_VIRTUAL(8);
    object->rectBottom                  = PIXEL_TO_VIRTUAL(EnemyTallFall.height);
    object->healthPoints                = 3;
    object->damage                      = 1;
    object->visible                     = 1;
    object->animationFrame              = 0;
    object->animationSubFrameCounter    = 0;
    object->invulnerableTimer           = 0;
    object->direction                   = 0;
    object->palette                     = PAL1;
    object->extra1                      = PIXEL_TO_VIRTUAL(-250);
    object->extra2 = RayCast(object->x, object->y);
    ComputeStandardActiveArea(object);


#ifdef DEBUG_OBJECTS
    KDebug_Alert("enemyCreate");
#endif
    
    return object;
}

void EnemyTallFallUpdateFallPhysics(ObjectStruct* object)
{
    object->speedy = 30;

    s16 startPosition = object->y + PIXEL_TO_VIRTUAL(object->animation->height);

    if (startPosition < 0)
    {
        startPosition = 0;
    }

    s16 loop;

    for (loop = startPosition; loop < startPosition + object->speedy; loop++)
    {
        if ((IsTileSolid_Virtual(object->x, loop) != 0))
        {
            object->y = loop - PIXEL_TO_VIRTUAL(object->animation->height);
            object->speedy = 0;
            object->animationFrame = 1;
            object->invulnerableTimer = SHAKE_TIME;
            ScrollManager_SetCameraShakeTime(SHAKE_TIME);

            object->aiState = ENEMY_AI_STATE_LANDED;

            EffectCreate(&DustImpact, 
                         &DustImpactVDPInfo, 
                         object->x - PIXEL_TO_VIRTUAL(DustImpact.width - 8), 
                         object->y + PIXEL_TO_VIRTUAL(object->animation->height) - PIXEL_TO_VIRTUAL(DustImpact.height),
                         0);
            EffectCreate(&DustImpact, 
                         &DustImpactVDPInfo, 
                         object->x + PIXEL_TO_VIRTUAL(EnemyTallFall.width - 8), 
                         object->y + PIXEL_TO_VIRTUAL(object->animation->height) - PIXEL_TO_VIRTUAL(DustImpact.height),
                         1);

            if (playerData.onGround)
            {
                PlayerManager_Stun();
            }

            break;
        }
    }

    object->y += object->speedy;
}

void EnemyTallFallUpdate(ObjectStruct* object)
{
    if (object->aiState == ENEMY_AI_STATE_WAIT)
    {
        if ((playerObject->x - object->x) > object->extra1)
        {
            object->aiState = ENEMY_AI_STATE_PREFALL;
            object->invulnerableTimer = PREFALL_TIME;
        }
    }
    else if (object->aiState == ENEMY_AI_STATE_PREFALL)
    {
        object->invulnerableTimer--;

        if (object->invulnerableTimer == 0)
        {
            object->aiState = ENEMY_AI_STATE_FALL;
        }
    }
    else
    {
        if (object->animationFrame == 0)
        {
           EnemyTallFallUpdateFallPhysics(object);
        }
        else
        {
            if (object->invulnerableTimer > 0)
            {
                object->invulnerableTimer--;

                if (object->invulnerableTimer == 0)
                {
                    EnemyTallWalkCreate(object->x, object->y);
                    object->objectState = OBJECTSTATE_INACTIVE;
                    object->aiState = ENEMY_AI_STATE_DEAD;
                }
            }
        }
    }

    object->spritedef.posx = VIRTUAL_TO_PIXEL(object->x) - foregroundScrollData.scrollx_vdp;
    object->spritedef.posy = VIRTUAL_TO_PIXEL(object->y) - foregroundScrollData.scrolly_vdp;
}

void EnemyTallFallDestroy(ObjectStruct* object)
{
#ifdef DEBUG_OBJECTS
    KDebug_Alert("EnemyTallDestroy");
#endif
}

void EnemyTallFallDraw(ObjectStruct* object)
{
    object->direction = (object->x > playerObject->x);

    GGAnimation_DrawObject(object);

    if (object->aiState == ENEMY_AI_STATE_FALL ||
        object->aiState == ENEMY_AI_STATE_PREFALL)
    {
        s16 spritey = VIRTUAL_TO_PIXEL(object->extra2) - foregroundScrollData.scrolly_vdp;

        GGAnimation_Draw(object->spritedef.posx - 4, 
                                   spritey,
                                   &Shadow, 
                                   0, 
                                   0,
                                   PAL0);
    }
}

void EnemyTallFallCollide(ObjectStruct* object, ObjectStruct* collidedObject)
{
}
