#include "EnemyTallWalk.h"

#include "ObjectCommon.h"

void EnemyTallWalkUpdate(ObjectStruct* object);
void EnemyTallWalkDestroy(ObjectStruct* object);
void EnemyTallWalkDraw(ObjectStruct* object);
void EnemyTallWalkCollide(ObjectStruct* object, ObjectStruct* collidedObject);


ObjectStruct* EnemyTallWalkCreate(s16 x, s16 y)
{
    ObjectStruct* object = ObjectManager_CreateObject(TYPE_ENEMY,
                                                      EnemyTallWalkUpdate, 
                                                      EnemyTallWalkDestroy, 
                                                      EnemyTallWalkDraw,
                                                      EnemyTallWalkCollide);

    if (!object)
    {
#ifdef DEBUG_OBJECTS
        KDebug_Alert("enemy could not be created!");
#endif
        return NULL;
    }

    object->x                           = x;
    object->y                           = y;
    object->speedx                      = 0;
    object->speedy                      = 0;
    object->animation                   = &EnemyTallWalkBottom;
    object->aiState                     = ENEMY_AI_STATE_ACTIVE;
    object->rectLeft                    = PIXEL_TO_VIRTUAL(0);
    object->rectRight                   = PIXEL_TO_VIRTUAL(EnemyTallWalkBottom.width);
    object->rectTop                     = PIXEL_TO_VIRTUAL(0);
    object->rectBottom                  = PIXEL_TO_VIRTUAL(64);
    object->healthPoints                = 2;
    object->damage                      = 1;
    object->visible                     = 1;
    object->animationFrame              = 0;
    object->animationSubFrameCounter    = 0;
    object->invulnerableTimer           = 0;
    ComputeStandardActiveArea(object);


    if (object->x < playerObject->x)
    {
        object->speedx = 1;
    }
    else
    {
        object->speedx = -1;
    }


#ifdef DEBUG_OBJECTS
    KDebug_Alert("enemyCreate");
#endif
    
    return object;
}

void EnemyTallWalkUpdateLeftRightPhysics(ObjectStruct* object)
{
    if (object->speedx > 0)
    {
        // if there's no tile at the foot of the enemy, turn around.
        if (IsTileSolid_Virtual(object->x + object->rectRight, object->y + object->rectBottom) == 0)
        {
            object->speedx = -object->speedx;
            goto updateobjectx;
        }

        s16 startXPosition = object->x + (object->rectRight + PIXEL_TO_VIRTUAL(1));
        s16 endXPosition = startXPosition + object->speedx;

        if (IsTileSolid_Virtual(endXPosition, object->y + (object->rectBottom - PIXEL_TO_VIRTUAL(1))) != 0)
        {
            endXPosition >>= 5;
            endXPosition <<= 5;

            object->x = endXPosition - (object->rectRight  + PIXEL_TO_VIRTUAL(1));
            object->speedx = -object->speedx;
            goto updateobjectx;
        }
    }
    else if (object->speedx < 0)
    {
        // if there's no tile at the foot of the enemy, turn around.
        if (IsTileSolid_Virtual(object->x + object->rectLeft, object->y + object->rectBottom) == 0)
        {
            object->speedx = -object->speedx;
            goto updateobjectx;
        }

        s16 startXPosition = object->x + (object->rectLeft - PIXEL_TO_VIRTUAL(1));
        s16 endXPosition = startXPosition + object->speedx;

        if (IsTileSolid_Virtual(endXPosition, object->y + (object->rectBottom - PIXEL_TO_VIRTUAL(1))) != 0)
        {
            startXPosition >>= 5;
            startXPosition <<= 5;
            object->x = startXPosition - (object->rectLeft - PIXEL_TO_VIRTUAL(1));
            object->speedx = -object->speedx;
            goto updateobjectx;
        }

    }

updateobjectx:
    object->x = object->x += object->speedx;
}

void EnemyTallWalkUpdate(ObjectStruct* object)
{
    if (object->aiState == ENEMY_AI_STATE_ACTIVE)
    {
        if (object->invulnerableTimer == 0)
        {
            EnemyTallWalkUpdateLeftRightPhysics(object);
        }
    }
    else if (object->aiState == ENEMY_AI_STATE_DEAD)
    {
        object->x += object->speedx;
        object->y += object->speedy >> 2;

        object->speedy++;

        if (object->visible)
        {
            if ((object->lifetime & 7) == 0)
            {
                ExplosionCreate(object->x, object->y);
            }
        }
        else
        {
            object->objectState = OBJECTSTATE_INACTIVE;
        }
    }

    object->spritedef.posx = VIRTUAL_TO_PIXEL(object->x) - foregroundScrollData.scrollx_vdp;
    object->spritedef.posy = VIRTUAL_TO_PIXEL(object->y) - foregroundScrollData.scrolly_vdp;

    if (object->invulnerableTimer > 0)
    {
        object->invulnerableTimer--;
    }
}

void EnemyTallWalkDestroy(ObjectStruct* object)
{
#ifdef DEBUG_OBJECTS
    KDebug_Alert("EnemyTallDestroy");
#endif
}

void EnemyTallWalkDraw(ObjectStruct* object)
{
    u16 palette = PAL1;

    if (object->invulnerableTimer > 0)
    {
        palette = PAL0;
    }

    GGAnimation_Draw(object->spritedef.posx - 8, 
                               object->spritedef.posy - (object->animationFrame & 1),
                               &EnemyTallWalkTop, 
                               0, 
                               (object->speedx < 0),
                               palette);



    GGAnimation_Draw(object->spritedef.posx, 
                               object->spritedef.posy + 32,
                               object->animation, 
                               object->animationFrame, 
                               (object->speedx < 0),
                               palette);

    object->animationSubFrameCounter++;

    if (object->animationSubFrameCounter > object->animation->frameDelayTime)
    {
        object->animationFrame++;
        object->animationSubFrameCounter = 0;
    }

    if (object->animationFrame > object->animation->numberOfFrames - 1)
    {
        object->animationFrame = 0;
    }

}

void EnemyTallWalkCollide(ObjectStruct* object, ObjectStruct* collidedObject)
{
    if (object->invulnerableTimer > 0)
    {
        return;
    }

    if (object->aiState != ENEMY_AI_STATE_ACTIVE)
    {
        return;
    }

    if (collidedObject->objectType != TYPE_PLAYER_PROJECTILE)
    {
        return;
    }

    object->healthPoints -= collidedObject->damage;

    object->invulnerableTimer = 10;

    if (object->healthPoints <= 0)
    {
        ExplosionCreate(object->x, object->y);
        object->aiState   = ENEMY_AI_STATE_DEAD;
        object->speedy    = 8;
        CoinCreate(object->x, object->y);
    }
}
