#include "Explosion.h"

#include "ObjectCommon.h"

void ExplosionUpdate(ObjectStruct* explosionObject);
void ExplosionDestroy(ObjectStruct* explosionObject);

ObjectStruct* ExplosionCreate(u16 x, u16 y)
{
    ObjectStruct* object = ObjectManager_CreateObject(TYPE_EFFECT,
                                                      ExplosionUpdate, 
                                                      ExplosionDestroy, 
                                                      GGAnimation_DrawObject,
                                                      CollisionFunctionStub);

    if (!object)
    {
#ifdef DEBUG_OBJECTS
        KDebug_Alert("Explosion could not be created!");
#endif
        return NULL;
    }

    object->x                           = x;
    object->y                           = y;
    object->animation                   = &LargeExplosion;
    object->animationFrame              = 0;
    object->animationSubFrameCounter    = 0;
    object->healthPoints                = 1;
    object->direction                   = 0;
    object->palette                     = PAL0;

    ComputeStandardActiveArea(object);

#ifdef DEBUG_OBJECTS
    KDebug_Alert("ExplosionCreate");
#endif

    return object;
}

void ExplosionUpdate(ObjectStruct* object)
{
    //KDebug_Alert("Explosion Update");

    object->animationSubFrameCounter++;

    if (object->animationSubFrameCounter > object->animation->frameDelayTime)
    {
        object->animationFrame++;
        object->animationSubFrameCounter = 0;
    }

    if (object->animationFrame > object->animation->numberOfFrames - 1)
    {
        object->objectState = OBJECTSTATE_INACTIVE;
        return;
    }

    object->spritedef.posx = VIRTUAL_TO_PIXEL(object->x) - foregroundScrollData.scrollx_vdp - (object->animation->width >> 1);
    object->spritedef.posy = VIRTUAL_TO_PIXEL(object->y) - foregroundScrollData.scrolly_vdp - (object->animation->height >> 1);
}

void ExplosionDestroy(ObjectStruct* object)
{
#ifdef DEBUG_OBJECTS
    KDebug_Alert("ExplosionDestroy");
#endif
}
