#include "MovingBlockHorizontal.h"

#include "MovingBlockBase.h"
#include "ObjectCommon.h"

void MovingBlockHorizontalUpdate(ObjectStruct* object);
void MovingBlockHorizontalCollide(ObjectStruct* object, ObjectStruct* collidedObject);

void CheckPlayerForSquish(s16 xPosition, s16 yPosition);

ObjectStruct* MovingBlockHorizontalCreate(s16 x, s16 y)
{
    ObjectStruct* object = ObjectManager_CreateObject(TYPE_PLATFORM,
                                                      MovingBlockHorizontalUpdate, 
                                                      MovingBlockBaseDestroy, 
                                                      GGAnimation_DrawObject,
                                                      MovingBlockHorizontalCollide);

    if (!object)
    {
#ifdef DEBUG_OBJECTS
        KDebug_Alert("MovingBlock could not be created!");
#endif
        return NULL;
    }

    object->x                           = x;
    object->y                           = y; // y spawn position is at the bottom of the sprite
    object->speedx                      = 8;
    object->speedy                      = 0;
    object->healthPoints                = 1;
    object->animation                   = &MovingBlock;
    object->rectLeft                    = PIXEL_TO_VIRTUAL(0);
    object->rectRight                   = PIXEL_TO_VIRTUAL(MovingBlock.width);
    object->rectTop                     = PIXEL_TO_VIRTUAL(0);
    object->rectBottom                  = PIXEL_TO_VIRTUAL(MovingBlock.height);
    object->visible                     = 1;
    object->animationFrame              = 0;
    object->animationSubFrameCounter    = 0;
    object->extra1                      = PIXEL_TO_VIRTUAL(104);
    object->extra2                      = 0;
    object->extra3                      = object->x; // original x
    object->extra4                      = object->y; // original y
    object->direction                   = 0;
    object->palette                     = PAL1;
    ComputeStandardActiveArea(object);

#ifdef DEBUG_OBJECTS
    KDebug_Alert("MovingBlockHorizontalCreate");
#endif
    
    return object;
}

void MovingBlockHorizontalUpdate(ObjectStruct* object)
{
    object->extra2 += object->speedx;
    if (mathutils_abs(object->extra2) > mathutils_abs(object->extra1))
    {
        object->speedx = -object->speedx; 
    }

    object->spritedef.posx = VIRTUAL_TO_PIXEL(object->extra3 + object->extra2) - foregroundScrollData.scrollx_vdp;
    object->spritedef.posy = VIRTUAL_TO_PIXEL(object->extra4) - foregroundScrollData.scrolly_vdp;

    object->x = object->extra3 + object->extra2;
    object->y = object->extra4;
}

void MovingBlockHorizontalCollide(ObjectStruct* object, ObjectStruct* collidedObject)
{
    if (playerData.platform == object ||
        playerData.platform != NULL)
    {
        return;
    }

    if (playerData.playerState == PLAYERSTATE_DEAD)
    {
        return;
    }

    s16 blockRectLeft = object->x + object->rectLeft;
    s16 blockRectTop = object->y + object->rectTop;
    s16 blockRectRight = object->x + object->rectRight;
    s16 blockRectBottom = object->y + object->rectBottom;

    s16 playerRectLeft = collidedObject->x + collidedObject->rectLeft;
    s16 playerRectTop = collidedObject->y + collidedObject->rectTop;
    s16 playerRectRight = collidedObject->x + collidedObject->rectRight;
    s16 playerRectBottom = collidedObject->y + collidedObject->rectBottom;

    s16 maxLeft = MAX(blockRectLeft, playerRectLeft);
    s16 maxTop = MAX(blockRectTop, playerRectTop);
    s16 minRight = MIN(blockRectRight, playerRectRight);
    s16 minBottom = MIN(blockRectBottom, playerRectBottom);

    s16 rectWidth = (minRight - maxLeft);
    s16 rectHeight = (minBottom - maxTop);

    if (rectWidth >= rectHeight) // push the player, up or down out of the block.
    {
        if (playerRectTop >= blockRectTop && playerRectTop <= blockRectBottom) 
        // the top part of the player is touching, so push down by the rectHeight.
        {
            collidedObject->y += rectHeight;

            if (playerData.playerState == PLAYERSTATE_JUMP)
            {
                PlayerManager_CallOnPhysicsCollisionFunction();
            }
        }
        else 
        // the bottom part of the player is touching so push up by the rectHeight
        // also, set ourselves on the platform.
        {
            if (playerData.playerState == PLAYERSTATE_FALL ||
                playerData.playerState == PLAYERSTATE_DOWNATTACK)
            {
                playerData.platform = object;

                collidedObject->y = object->y - PIXEL_TO_VIRTUAL(32);

                PlayerManager_CallOnPhysicsCollisionFunction();
            }
        }
    }
    else // push the player left or right out of the block
    {
        // player being pushed right
        if (playerRectLeft >= blockRectLeft && playerRectLeft <= blockRectRight)
        {
            collidedObject->x += rectWidth;
            collidedObject->speedx = 0;
            CheckPlayerForSquish(VIRTUAL_TO_PIXEL(playerObject->x) + 4, VIRTUAL_TO_PIXEL(playerObject->y));
        }
        else
        // player being pushed left
        {
            collidedObject->x -= rectWidth;
            collidedObject->speedx = 0;
            CheckPlayerForSquish(VIRTUAL_TO_PIXEL(playerObject->x) - 4, VIRTUAL_TO_PIXEL(playerObject->y));
        }
    }
}

void CheckPlayerForSquish(s16 xPosition, s16 yPosition)
{
    if ((IsTileSolid_Pixel(xPosition, yPosition + 8) != 0) ||
        (IsTileSolid_Pixel(xPosition, yPosition + 16) != 0) ||
        (IsTileSolid_Pixel(xPosition, yPosition + 24) != 0) ||
        (IsTileSolid_Pixel(xPosition, yPosition + 31) != 0))
    {
        PlayerManager_Die();
    }
}

