#include "MovingBlockVertical.h"

#include "MovingBlockBase.h"
#include "ObjectCommon.h"

void MovingBlockVerticalUpdate(ObjectStruct* object);
void MovingBlockVerticalCollide(ObjectStruct* object, ObjectStruct* collidedObject);

ObjectStruct* MovingBlockVerticalCreate(s16 x, s16 y)
{
    ObjectStruct* object = ObjectManager_CreateObject(TYPE_PLATFORM,
                                                      MovingBlockVerticalUpdate, 
                                                      MovingBlockBaseDestroy, 
                                                      GGAnimation_DrawObject,
                                                      MovingBlockVerticalCollide);

    if (!object)
    {
#ifdef DEBUG_OBJECTS
        KDebug_Alert("MovingBlock could not be created!");
#endif
        return NULL;
    }

    object->x                           = x;
    object->y                           = y;
    object->speedx                      = 0;
    object->speedy                      = 4;
    object->healthPoints                = 1;
    object->animation                   = &MovingBlock;
    object->rectLeft                    = PIXEL_TO_VIRTUAL(0);
    object->rectRight                   = PIXEL_TO_VIRTUAL(MovingBlock.width);
    object->rectTop                     = PIXEL_TO_VIRTUAL(0);
    object->rectBottom                  = PIXEL_TO_VIRTUAL(MovingBlock.height);
    object->visible                     = 1;
    object->animationFrame              = 0;
    object->animationSubFrameCounter    = 0;
    object->extra1                      = PIXEL_TO_VIRTUAL(32);
    object->extra2                      = 0;
    object->extra3                      = object->x; // original x
    object->extra4                      = object->y; // original y
    object->direction                   = 0;
    object->palette                     = PAL1;
    ComputeStandardActiveArea(object);

#ifdef DEBUG_OBJECTS
    KDebug_Alert("MovingBlockVerticalCreate");
#endif
    
    return object;
}

void MovingBlockVerticalUpdate(ObjectStruct* object)
{
    object->extra2 += object->speedy;
    if (mathutils_abs(object->extra2) > mathutils_abs(object->extra1))
    {
        object->speedy = -object->speedy; 
    }


    object->spritedef.posx = VIRTUAL_TO_PIXEL(object->extra3) - foregroundScrollData.scrollx_vdp;
    object->spritedef.posy = VIRTUAL_TO_PIXEL(object->extra4 + object->extra2) - foregroundScrollData.scrolly_vdp;

    object->x = object->extra3;
    object->y = object->extra4 + object->extra2;
}


void MovingBlockVerticalCollide(ObjectStruct* object, ObjectStruct* collidedObject)
{
    if (playerData.platform == object ||
        playerData.platform != NULL)
    {
        return;
    }

    if (playerData.playerState == PLAYERSTATE_DEAD)
    {
        return;
    }

    s16 blockRectLeft = object->x + object->rectLeft;
    s16 blockRectTop = object->y + object->rectTop;
    s16 blockRectRight = object->x + object->rectRight;
    s16 blockRectBottom = object->y + object->rectBottom;

    s16 playerRectLeft = collidedObject->x + collidedObject->rectLeft;
    s16 playerRectTop = collidedObject->y + collidedObject->rectTop;
    s16 playerRectRight = collidedObject->x + collidedObject->rectRight;
    s16 playerRectBottom = collidedObject->y + collidedObject->rectBottom;

    s16 maxLeft = MAX(blockRectLeft, playerRectLeft);
    s16 maxTop = MAX(blockRectTop, playerRectTop);
    s16 minRight = MIN(blockRectRight, playerRectRight);
    s16 minBottom = MIN(blockRectBottom, playerRectBottom);

    s16 rectWidth = (minRight - maxLeft);
    s16 rectHeight = (minBottom - maxTop);

    if (rectWidth >= rectHeight) // push the player, up or down out of the block.
    {
        if (playerRectTop >= blockRectTop && playerRectTop <= blockRectBottom) 
        // the top part of the player is touching, so push down by the rectHeight.
        {
            collidedObject->y += rectHeight;

            if (playerData.playerState == PLAYERSTATE_JUMP)
            {
                PlayerManager_CallOnPhysicsCollisionFunction();
            }


            if (object->speedy > 0)
            {
                if (playerData.playerState == PLAYERSTATE_IDLE ||
                    playerData.playerState == PLAYERSTATE_RUN)
                {
                    PlayerManager_Die();
                }
            }
        }
        else 
        // the bottom part of the player is touching so push up by the rectHeight
        // also, set ourselves on the platform.
        {
            if (playerData.playerState == PLAYERSTATE_FALL ||
                playerData.playerState == PLAYERSTATE_DOWNATTACK)
            {
                playerData.platform = object;

                collidedObject->y = object->y - PIXEL_TO_VIRTUAL(32);

                PlayerManager_CallOnPhysicsCollisionFunction();
            }
        }
    }
    else // push the player left or right out of the block
    {
        if (playerRectLeft >= blockRectLeft && playerRectLeft <= blockRectRight)
        {
            collidedObject->x += rectWidth;
            collidedObject->speedx = 0;
        }
        else
        {
            collidedObject->x -= rectWidth;
            collidedObject->speedx = 0;
        }
    }
}
