#include "SwordDamage.h"

#include "ObjectCommon.h"

void SwordDamageUpdate(ObjectStruct* bulletObject);
void SwordDamageDestroy(ObjectStruct* bulletObject);
void SwordDamageDraw(ObjectStruct* bulletObject);
void SwordDamageCollide(ObjectStruct* bulletObject, ObjectStruct* collidedObject);


ObjectStruct* SwordDamageCreate(s16 x, s16 y, s16 width, s16 height, u16 swordDamageType)
{
    ObjectStruct* object = ObjectManager_CreateObject(TYPE_PLAYER_PROJECTILE,
                                                   SwordDamageUpdate, 
                                                   SwordDamageDestroy, 
                                                   SwordDamageDraw,
                                                   SwordDamageCollide);

    if (!object)
    {
#ifdef DEBUG_OBJECTS
        KDebug_Alert("SwordDamage could not be created!");
#endif
        return NULL;
    }

    object->x                = x;
    object->y                = y;
    object->speedx           = 0;
    object->speedy           = 0;
    object->rectLeft         = PIXEL_TO_VIRTUAL(0);
    object->rectRight        = PIXEL_TO_VIRTUAL(width);
    object->rectTop          = PIXEL_TO_VIRTUAL(0);
    object->rectBottom       = PIXEL_TO_VIRTUAL(height);
    object->damage           = 1;
    object->healthPoints     = 1;
    object->visible          = 1;
    object->lifetime         = 0;
    object->aiState          = swordDamageType;
    ComputeStandardActiveArea(object);

#ifdef DEBUG_OBJECTS
    KDebug_Alert("SwordDamageCreate");
#endif

    return object;
}

void SwordDamageUpdate(ObjectStruct* object)
{
    object->spritedef.posx = VIRTUAL_TO_PIXEL(object->x) - foregroundScrollData.scrollx_vdp;
    object->spritedef.posy = VIRTUAL_TO_PIXEL(object->y) - foregroundScrollData.scrolly_vdp;
   
    if (object->lifetime > 0)
    {
        object->objectState = OBJECTSTATE_INACTIVE;
    }
}

void SwordDamageDestroy(ObjectStruct* object)
{
#ifdef DEBUG_OBJECTS
    KDebug_Alert("SwordDamageDestroy");
#endif
}

void SwordDamageDraw(ObjectStruct* object)
{
    if (debugSettings.drawSwordDamage == 1)
    {
        MyVDP_setSprite(object->spritedef.posx, object->spritedef.posy, SPRITE_SIZE(1,1), 1);

        MyVDP_setSprite(object->spritedef.posx - 8 + VIRTUAL_TO_PIXEL(object->rectRight), 
                        object->spritedef.posy - 8 + VIRTUAL_TO_PIXEL(object->rectBottom), SPRITE_SIZE(1,1), 1);

        MyVDP_setSprite(object->spritedef.posx, 
                        object->spritedef.posy - 8 + VIRTUAL_TO_PIXEL(object->rectBottom), SPRITE_SIZE(1,1), 1);

        MyVDP_setSprite(object->spritedef.posx - 8 + VIRTUAL_TO_PIXEL(object->rectRight), 
                        object->spritedef.posy, SPRITE_SIZE(1,1), 1);

        /*
        MyVDP_setSprite(object->spritedef.posx, object->spritedef.posy, SPRITE_SIZE(1,1), 1);

        if (object->aiState != SWORDDAMAGE_DOWNATTACK)
        {
            MyVDP_setSprite(object->spritedef.posx + 8, object->spritedef.posy, SPRITE_SIZE(1,1), 1);
            MyVDP_setSprite(object->spritedef.posx + 16, object->spritedef.posy, SPRITE_SIZE(1,1), 1);
        }
        */
    }
}

void SwordDamageCollide(ObjectStruct* object, ObjectStruct* collidedObject)
{
    

    if (object->aiState == SWORDDAMAGE_DOWNATTACK)
    {
        playerData.freezeTimer = 5;
        playerData.onfreezeTimerDone = PlayerManager_Jump;
    }
    else
    {
        playerData.freezeTimer = 3;
    }
}

